@@ -3,19 +3,25 @@ import sys
 from robotframework_ls.constants import NULL
 from robocorp_ls_core.robotframework_log import get_logger
 import threading
-from typing import Optional, Dict
-from robocorp_ls_core.protocols import Sentinel
+from typing import Optional, Dict, Set, Iterator
+from robocorp_ls_core.protocols import Sentinel, IEndPoint
 from robotframework_ls.impl.protocols import ILibraryDoc
 import itertools
 from robocorp_ls_core.watchdog_wrapper import IFSObserver
+from robotframework_ls.impl.robot_lsp_constants import (
+    OPTION_ROBOT_LIBRARIES_LIBDOC_NEEDS_ARGS,
+)
+from functools import lru_cache
+from robotframework_ls.impl.libspec_warmup import (
+    _norm_filename,
+    _normfile,
+    LibspecWarmup,
+)
+from pathlib import Path
 
 log = get_logger(__name__)
 
 
-def _normfile(filename):
-    return os.path.abspath(os.path.normpath(os.path.normcase(filename)))
-
-
 def _get_libspec_mutex_name(libspec_filename):
     from robocorp_ls_core.system_mutex import generate_mutex_name
 
@@ -30,6 +36,13 @@ def _get_additional_info_filename(spec_filename):
     return additional_info_filename
 
 
+@lru_cache(maxsize=250)
+def _get_digest_from_string(s: str) -> str:
+    import hashlib
+
+    return hashlib.sha256(s.encode("utf-8", "replace")).hexdigest()[:8]
+
+
 def _load_library_doc_and_mtime(spec_filename, obtain_mutex=True):
     """
     :param obtain_mutex:
@@ -227,10 +240,6 @@ class _LibInfo(object):
         return True
 
 
-def _norm_filename(path):
-    return os.path.normcase(os.path.realpath(os.path.abspath(path)))
-
-
 class _FolderInfo(object):
     def __init__(self, folder_path, recursive):
         self.folder_path = folder_path
@@ -404,15 +413,27 @@ class LibspecManager(object):
 
     def __init__(
         self,
-        builtin_libspec_dir=None,
-        user_libspec_dir=None,
+        builtin_libspec_dir: Optional[str] = None,
+        user_libspec_dir: Optional[str] = None,
+        dir_cache_dir: Optional[str] = None,
         observer: Optional[IFSObserver] = None,
+        endpoint: Optional[IEndPoint] = None,
+        pre_generate_libspecs: bool = False,
     ):
         """
         :param __internal_libspec_dir__:
             Only to be used in tests (to regenerate the builtins)!
         """
         from robocorp_ls_core import watchdog_wrapper
+        from robocorp_ls_core.cache import DirCache
+        from robotframework_ls import robot_config
+
+        dir_cache = DirCache(
+            dir_cache_dir
+            or os.path.join(robot_config.get_robotframework_ls_home(), ".cache")
+        )
+        self.pre_generate_libspecs = pre_generate_libspecs
+        self._libspec_warmup = LibspecWarmup(endpoint, dir_cache)
 
         self._libspec_failures_cache: Dict[tuple, bool] = {}
 
@@ -459,11 +480,25 @@ class LibspecManager(object):
 
         # Spec info found in the pythonpath
         pythonpath_folder_to_folder_info: Dict[str, _FolderInfo] = {}
+        found = set()
         for path in sys.path:
-            if path and os.path.isdir(path):
-                pythonpath_folder_to_folder_info[path] = _FolderInfo(
-                    path, recursive=False
-                )
+            if path:
+                try:
+                    resolved = Path(path).resolve()  # also solves links.
+                except:
+                    log.exception(
+                        f"Unable to Path.resolve({path!r}) (resolving PYTHONPATH entries)."
+                    )
+                    continue
+
+                if resolved in found:
+                    continue
+                found.add(resolved)
+                if os.path.isdir(path):
+                    pythonpath_folder_to_folder_info[path] = _FolderInfo(
+                        path, recursive=False
+                    )
+
         self._pythonpath_folder_to_folder_info: Dict[
             str, _FolderInfo
         ] = pythonpath_folder_to_folder_info
@@ -481,8 +516,10 @@ class LibspecManager(object):
         # Must be set from the outside world when needed.
         self.config = None
 
-        log.debug("Generating builtin libraries.")
-        self._gen_builtin_libraries()
+        if self.pre_generate_libspecs:
+            log.debug("Generating builtin libraries libspec.")
+            self._libspec_warmup.gen_builtin_libraries(self)
+
         log.debug("Synchronizing internal caches.")
         self._synchronize()
         log.debug("Finished initializing LibspecManager.")
@@ -504,6 +541,11 @@ class LibspecManager(object):
 
     @config.setter
     def config(self, config):
+        from robotframework_ls.impl.robot_lsp_constants import (
+            OPTION_ROBOT_LIBRARIES_LIBDOC_PRE_GENERATE,
+        )
+        from robocorp_ls_core.basic import make_unique
+
         self._check_in_main_thread()
         from robotframework_ls.impl.robot_lsp_constants import OPTION_ROBOT_PYTHONPATH
 
@@ -520,6 +562,15 @@ class LibspecManager(object):
                 if old_entry not in pythonpath_entries:
                     self.remove_additional_pythonpath_folder(old_entry)
 
+            pre_generate = []
+            pre_generate.extend(
+                config.get_setting(OPTION_ROBOT_LIBRARIES_LIBDOC_PRE_GENERATE, list, [])
+            )
+
+            if self.pre_generate_libspecs:
+                log.debug("Generating user/pythonpath libraries libspec.")
+                self._libspec_warmup.gen_user_libraries(self, make_unique(pre_generate))
+
         self.synchronize_additional_pythonpath_folders()
 
     @property
@@ -600,63 +651,6 @@ class LibspecManager(object):
         else:
             log.debug("Additional pythonpath folder already removed: %s", folder_path)
 
-    def _gen_builtin_libraries(self):
-        """
-        Generates .libspec files for the libraries builtin (if needed).
-        """
-
-        try:
-            import time
-            from concurrent import futures
-            from robotframework_ls.impl import robot_constants
-            from robocorp_ls_core.system_mutex import timed_acquire_mutex
-            from robocorp_ls_core.system_mutex import generate_mutex_name
-            from robotframework_ls.impl.robot_constants import RESERVED_LIB
-
-            initial_time = time.time()
-            wait_for = []
-
-            max_workers = min(10, (os.cpu_count() or 1) + 4)
-            thread_pool = futures.ThreadPoolExecutor(max_workers=max_workers)
-
-            try:
-                log.debug("Waiting for mutex to generate builtins.")
-                with timed_acquire_mutex(
-                    generate_mutex_name(
-                        _norm_filename(self._builtins_libspec_dir),
-                        prefix="gen_builtins_",
-                    ),
-                    timeout=100,
-                ):
-                    log.debug("Obtained mutex to generate builtins.")
-                    for libname in robot_constants.STDLIBS:
-                        if libname == RESERVED_LIB:
-                            continue
-                        builtins_libspec_dir = self._builtins_libspec_dir
-                        if not os.path.exists(
-                            os.path.join(builtins_libspec_dir, f"{libname}.libspec")
-                        ):
-                            wait_for.append(
-                                thread_pool.submit(
-                                    self._create_libspec, libname, is_builtin=True
-                                )
-                            )
-                    for future in wait_for:
-                        future.result()
-
-                if wait_for:
-                    log.debug(
-                        "Total time to generate builtins: %.2fs"
-                        % (time.time() - initial_time)
-                    )
-                    self.synchronize_internal_libspec_folders()
-            finally:
-                thread_pool.shutdown(wait=False)
-        except:
-            log.exception("Error creating builtin libraries.")
-        finally:
-            log.info("Finished creating builtin libraries.")
-
     def synchronize_workspace_folders(self):
         for folder_info in self._workspace_folder_uri_to_folder_info.values():
             folder_info.start_watch(self._fs_observer, self._file_changes_notifier)
@@ -690,6 +684,16 @@ class LibspecManager(object):
         self.synchronize_additional_pythonpath_folders()
         self.synchronize_internal_libspec_folders()
 
+    def collect_all_tracked_folders(self) -> Iterator[str]:
+        from robocorp_ls_core import uris
+
+        for uri in self._workspace_folder_uri_to_folder_info.keys():
+            yield uris.to_fs_path(uri)
+
+        yield from self._pythonpath_folder_to_folder_info.keys()
+
+        yield from self._additional_pythonpath_folder_to_folder_info.keys()
+
     def iter_lib_info(self, builtin=False):
         """
         :rtype: generator(_LibInfo)
@@ -749,24 +753,33 @@ class LibspecManager(object):
         )
 
     def _create_libspec(
-        self, libname, *, is_builtin=False, target_file: Optional[str] = None
+        self,
+        libname,
+        *,
+        is_builtin=False,
+        target_file: Optional[str] = None,
+        args: Optional[str] = None,
     ):
         """
         :param target_file:
             If given this is the library file (i.e.: c:/foo/bar.py) which is the
             actual library we're creating the spec for.
         """
-        cache_key = (libname, is_builtin, target_file)
-        not_created = self._libspec_failures_cache.get(cache_key, Sentinel.SENTINEL)
-        if not_created is Sentinel.SENTINEL:
-            created = self._cached_create_libspec(libname, is_builtin, target_file)
+        cache_key = (libname, is_builtin, target_file, args)
+        failed_to_create_previously = self._libspec_failures_cache.get(
+            cache_key, Sentinel.SENTINEL
+        )
+        if failed_to_create_previously is Sentinel.SENTINEL:
+            created = self._cached_create_libspec(
+                libname, is_builtin, target_file, args
+            )
             if not created:
                 # Always set as a whole (to avoid racing conditions).
                 cp = self._libspec_failures_cache.copy()
                 cp[cache_key] = False
                 self._libspec_failures_cache = cp
 
-        return not_created
+        return failed_to_create_previously
 
     def _subprocess_check_output(self, *args, **kwargs):
         # Only done for mocking.
@@ -776,9 +789,10 @@ class LibspecManager(object):
 
     def _cached_create_libspec(
         self,
-        libname,
+        libname: str,
         is_builtin: bool,
         target_file: Optional[str],
+        args: Optional[str],
         *,
         _internal_force_text=False,  # Should only be set from within this function.
     ):
@@ -787,7 +801,6 @@ class LibspecManager(object):
         :raise Exception: if unable to create the library.
         """
         import time
-        from robotframework_ls.impl import robot_constants
         from robocorp_ls_core.subprocess_wrapper import subprocess
         from robocorp_ls_core.system_mutex import timed_acquire_mutex
         from robocorp_ls_core.robotframework_log import get_log_level
@@ -846,21 +859,10 @@ class LibspecManager(object):
                     if os.path.exists(entry):
                         call.extend(["-P", entry])
 
-                call.append(libname)
-                libspec_dir = self._user_libspec_dir
-                if libname in robot_constants.STDLIBS:
-                    libspec_dir = self._builtins_libspec_dir
-
-                if target_file:
-                    import hashlib
-
-                    digest = hashlib.sha256(
-                        target_file.encode("utf-8", "replace")
-                    ).hexdigest()[:8]
-
-                    libspec_filename = os.path.join(libspec_dir, digest + ".libspec")
-                else:
-                    libspec_filename = os.path.join(libspec_dir, libname + ".libspec")
+                call.append("::".join([libname, args] if args else [libname]))
+                libspec_filename = self._compute_libspec_filename(
+                    libname, is_builtin, target_file, args
+                )
 
                 log.debug(f"Obtaining mutex to generate libspec: {libspec_filename}.")
                 with timed_acquire_mutex(
@@ -922,6 +924,7 @@ class LibspecManager(object):
                                     libname,
                                     is_builtin,
                                     target_file,
+                                    args,
                                     _internal_force_text=True,
                                 )
 
@@ -947,9 +950,42 @@ class LibspecManager(object):
     def dispose(self):
         self._file_changes_notifier.dispose()
 
-    def _do_create_libspec_on_get(self, libname, target_file: Optional[str]):
+    def _compute_libspec_filename(
+        self,
+        libname: str,
+        is_builtin: bool = False,
+        target_file: Optional[str] = None,
+        args: Optional[str] = None,
+    ):
+        from robotframework_ls.impl import robot_constants
 
-        if self._create_libspec(libname, target_file=target_file):
+        libspec_dir = self._user_libspec_dir
+        if libname in robot_constants.STDLIBS:
+            libspec_dir = self._builtins_libspec_dir
+
+        if target_file:
+            if args:
+                digest = (
+                    _get_digest_from_string(target_file)
+                    + "_"
+                    + _get_digest_from_string(args)
+                )
+            else:
+                digest = _get_digest_from_string(target_file)
+
+            libspec_filename = os.path.join(libspec_dir, digest + ".libspec")
+        elif not args:
+            libspec_filename = os.path.join(libspec_dir, libname + ".libspec")
+        else:
+            digest = _get_digest_from_string(args)
+            libspec_filename = os.path.join(libspec_dir, libname + digest + ".libspec")
+        return libspec_filename
+
+    def _do_create_libspec_on_get(
+        self, libname, target_file: Optional[str], args: Optional[str]
+    ):
+
+        if self._create_libspec(libname, target_file=target_file, args=args):
             self.synchronize_internal_libspec_folders()
             return True
         return False
@@ -1005,6 +1041,7 @@ class LibspecManager(object):
         create: bool = True,
         current_doc_uri: Optional[str] = None,
         builtin: bool = False,
+        args: Optional[str] = None,
     ) -> Optional[ILibraryDoc]:
         """
         :param libname:
@@ -1018,6 +1055,27 @@ class LibspecManager(object):
         target_file: str = ""
         normalized_target_file: str = ""
 
+        config = self.config
+        libraries_libdoc_needs_args_lower: Set[str]
+        if config is not None:
+            libraries_libdoc_needs_args_lower = set(
+                str(x).lower()
+                for x in config.get_setting(
+                    OPTION_ROBOT_LIBRARIES_LIBDOC_NEEDS_ARGS,
+                    list,
+                    ["remote", "fakerlib"],
+                )
+            )
+        else:
+            libraries_libdoc_needs_args_lower = {"remote", "fakerlib"}
+
+        # Note that experimentally, using '*' may pass args to all libraries.
+        if (
+            libname_lower not in libraries_libdoc_needs_args_lower
+            and "*" not in libraries_libdoc_needs_args_lower
+        ):
+            args = None
+
         if not builtin:
             found_target_filename = self._get_library_target_filename(
                 libname, current_doc_uri
@@ -1026,6 +1084,12 @@ class LibspecManager(object):
                 target_file = found_target_filename
                 normalized_target_file = os.path.normcase(os.path.normpath(target_file))
 
+        if libname_lower.endswith((".py", ".class", ".java")):
+            libname_lower = os.path.splitext(libname_lower)[0]
+
+        if "/" in libname_lower or "\\" in libname_lower:
+            libname_lower = os.path.basename(libname_lower)
+
         lib_info: _LibInfo
         for lib_info in self.iter_lib_info(builtin=builtin):
             library_doc = lib_info.library_doc
@@ -1035,27 +1099,39 @@ class LibspecManager(object):
             # Note: this is only valid for the cases where we can regenerate the info
             # for cases where this information is builtin, only match by the name.
             if target_file and lib_info._can_regenerate:
-                found = (
-                    library_doc.source
-                    and os.path.normcase(os.path.normpath(library_doc.source))
-                    == normalized_target_file
-                )
-                if not found:
-                    try:
-                        found = library_doc.source and os.path.samefile(
-                            library_doc.source, target_file
-                        )
-                    except:
-                        # os.path.samefile touches the filesystem, so, it can
-                        # raise an exception.
-                        found = False
-            else:
-                if libname_lower.endswith((".py", ".class", ".java")):
-                    libname_lower = os.path.splitext(libname_lower)[0]
+                if args:
+                    digest = (
+                        _get_digest_from_string(target_file)
+                        + "_"
+                        + _get_digest_from_string(args)
+                    )
+                    found = library_doc.filename.endswith(digest + ".libspec")
 
-                if "/" in libname_lower or "\\" in libname_lower:
-                    libname_lower = os.path.basename(libname_lower)
-                found = library_doc.name and library_doc.name.lower() == libname_lower
+                else:
+                    found = (
+                        library_doc.source
+                        and os.path.normcase(os.path.normpath(library_doc.source))
+                        == normalized_target_file
+                    )
+                    if not found:
+                        try:
+                            found = library_doc.source and os.path.samefile(
+                                library_doc.source, target_file
+                            )
+                        except:
+                            # os.path.samefile touches the filesystem, so, it can
+                            # raise an exception.
+                            found = False
+            else:
+                if not args:
+                    found = (
+                        library_doc.name and library_doc.name.lower() == libname_lower
+                    )
+                else:
+                    digest = _get_digest_from_string(args)
+                    found = library_doc.filename.endswith(
+                        os.path.normcase(libname + digest + ".libspec")
+                    )
 
             if found:
                 if not lib_info.verify_sources_sync():
@@ -1063,7 +1139,7 @@ class LibspecManager(object):
                         # Found but it's not in sync. Try to regenerate (don't proceed
                         # because we don't want to match a lower priority item, so,
                         # regenerate and get from the cache without creating).
-                        self._do_create_libspec_on_get(libname, target_file)
+                        self._do_create_libspec_on_get(libname, target_file, args)
 
                         # Note: get even if it if was not created (we may match
                         # a lower priority library).
@@ -1072,6 +1148,7 @@ class LibspecManager(object):
                             create=False,
                             current_doc_uri=current_doc_uri,
                             builtin=builtin,
+                            args=args,
                         )
                     else:
                         # Not in sync and it should not be created, just skip it.
@@ -1080,12 +1157,13 @@ class LibspecManager(object):
                     return library_doc
 
         if create:
-            if self._do_create_libspec_on_get(libname, target_file):
+            if self._do_create_libspec_on_get(libname, target_file, args):
                 return self.get_library_info(
                     libname,
                     create=False,
                     current_doc_uri=current_doc_uri,
                     builtin=builtin,
+                    args=args,
                 )
 
         log.debug("Unable to find library named: %s", libname)
