@@ -88,7 +88,9 @@ class Config(object):
             else:
                 log.debug("Unable to make workspace replacement for variable: %s", name)
 
-        elif (name.startswith("${env.") or name.startswith("${env:")) and name.endswith("}"):
+        elif (name.startswith("${env.") or name.startswith("${env:")) and name.endswith(
+            "}"
+        ):
             name = name[6:-1]
             ret = os.environ.get(name)  # Note: should be case-insensitive on windows.
         else:
@@ -125,7 +127,7 @@ class Config(object):
         :param settings:
             The settings where the variables should be replaced.
             Note that this instance is unchanged.
-            
+
         :return dict:
             Returns a new dict with the variables replaced.
         """
@@ -168,6 +170,16 @@ class Config(object):
         self._override_settings = self._replace_variables_in_settings(settings)
         self._update_full_settings()
 
+    @implements(IConfig.update_override_settings)
+    def update_override_settings(self, override_settings):
+        settings = flatten_keys(override_settings, all_options=self.ALL_OPTIONS)
+        original = self._original_override_settings.copy()
+        original.update(settings)
+
+        self._original_override_settings = original
+        self._override_settings = self._replace_variables_in_settings(original)
+        self._update_full_settings()
+
     @implements(IConfig.get_full_settings)
     def get_full_settings(self):
         return self._full_settings
